import controlP5.*;

// A basic Point class, as we saw during lectures.  The only
// extension is an "add" method that consumes a second Point as
// input and produces the sum of the two points.  So you can write
// something like this:
//
// Point p1 = new Point( 4, 5 );
// Point p2 = new Point( 7, -1 );
// Point p3 = p1.add( p2 ); // p3 is now ( 11, 4 )
//
// You shouldn't have to change this class at all.  Just leave it 
// as-is.
//
class Point
{
  float x;
  float y;

  Point( float x, float y )
  {
    this.x = x;
    this.y = y;
  }

  Point add( Point other )
  {
    return new Point( x + other.x, y + other.y );
  }
}

// A convenience function to avoid any unnecessary suffering over
// trigonometry.  Convert a distance/angle pair (i.e., a point 
// given in polar coordinates) into usual x/y coordinates.
Point makePolar( float distance, float angle )
{
  return new Point(
  distance * cos( angle ), distance * sin( angle ) );
}

//
// PUT YOUR Turtle CLASS HERE!!
//

// You shouldn't change any code below this point.

// A few global variables.  The global Turtle object is obvious.
// Also use a PGraphics to store an off-screen buffer of the image
// being drawn, and declare some ControlP5 junk.
Turtle turtle;
PGraphics graphics;

ControlP5 cp5;
Textfield txt;

// Execute one Logo statement.  Basically just break the statement
// into parts and check which command you want to use by looking at
// the first part.  May as well be case-insensitive by using
// the toLowerCase() method of the String class.
void execute( String cmd )
{
  String[] wds = split( cmd, ' ' );
  String wd = wds[0].toLowerCase();
  if ( wd.equals( "fd" ) ) {
    turtle.forward( float( wds[1] ) );
  } else if ( wd.equals( "bk" ) ) {
    turtle.back( float( wds[1] ) );
  } else if ( wd.equals( "lt" ) ) {
    turtle.left( float( wds[1] ) );
  } else if ( wd.equals( "rt" ) ) {
    turtle.right( float( wds[1] ) );
  } else if ( wd.equals( "pu" ) ) {
    turtle.penUp();
  } else if ( wd.equals( "pd" ) ) {
    turtle.penDown();
  } else if( wd.equals( "run" ) ) {
    String[] lines = loadStrings( wds[1] );
    for( int idx = 0; idx < lines.length; ++idx ) {
      execute( lines[idx] );
    }
  } else if( wd.equals( "reset" ) ) {
    graphics.background( 0 );
    turtle = new Turtle();
  }
}

void setup()
{
  size( 500, 500 );
  
  turtle = new Turtle();
  graphics = createGraphics( width, height );
  graphics.beginDraw();
  graphics.background( 0, 255 );
  graphics.stroke( 255 );
  graphics.strokeWeight( 1.0 );
  graphics.noFill();
  graphics.endDraw();
  
  cp5 = new ControlP5( this );
  txt = cp5.addTextfield( "Command" )
    .setPosition( 0, height - 30 )
    .setSize( width, 30 )
    .setFont( new ControlFont( createFont( "Helvetica", 20, true ) ) );
}

void draw()
{
  // Paste the current drawing onto the screen.
  image( graphics, 0, 0 );
  
  // Ask the turtle to draw a representation of itself, so 
  // that we can track it on the screen.
  fill( 255 );
  pushMatrix();
  translate( 250, 250 );
  turtle.draw();
  popMatrix();
}

// With the interface we have this only happens if the user
// presses enter after typing a command into the textfield at 
// the bottom of the sketch.  In that case execute the
// statement they typed in.
void controlEvent( ControlEvent ev )
{
  String cmd = txt.getText();
  
  // This is a bit sneaky, and uses hidden features of Processing.
  // If you care (and you don't have to), I temporarily overwrite
  // the internal PGraphics object (called "g"), which is the target
  // for all built-in graphics functions like pushMatrix(), line(),
  // fill(), etc.  By doing this, I redirect the Turtle's drawing
  // commands so they silently get sucked into the offscreen graphics
  // buffer instead of going straight onto the screen where they'd
  // eventually be erased.  When you're writing your Turtle class,
  // you shouldn't notice any difference.
  
  // Create a temporary variable to hold the current PGraphics instance.
  PGraphics g_tmp = g;
  // Now overwrite it with *our* PGraphics.
  g = graphics;
  
  // OK, safe to set up a geometric context and execute the current
  // statement.
  graphics.beginDraw();
  pushMatrix();
  // translate so that (0,0) is at the centre of the sketch window.
  translate( 250, 250 );
  execute( cmd );
  popMatrix();
  graphics.endDraw();
  
  // Finally, restore the original PGraphics object.
  g = g_tmp;
}
